<?php
namespace Drupal\physical_quantity_fields\Plugin\Field\FieldWidget;

use Drupal\Core\Annotation\Translation;
use Drupal\Core\Field\Annotation\FieldWidget;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Field\WidgetInterface;
use Drupal\Core\Form\FormStateInterface;

/**
 * Plugin implementation of the 'physical_quantity_mass_default_widget' widget.
 *
 * @FieldWidget(
 *   id = "physical_quantity_mass_default_widget",
 *   label = @Translation("Default Widget"),
 *   field_types = {
 *     "physical_quantity_mass"
 *   }
 * )
 */

class MassDefaultWidget extends WidgetBase implements WidgetInterface  {
    /**
     * Returns the form for a single field widget.
     *
     * Field widget form elements should be based on the passed-in $element, which
     * contains the base form element properties derived from the field
     * configuration.
     *
     * The BaseWidget methods will set the weight, field name and delta values for
     * each form element. If there are multiple values for this field, the
     * formElement() method will be called as many times as needed.
     *
     * Other modules may alter the form element provided by this function using
     * hook_field_widget_form_alter() or
     * hook_field_widget_WIDGET_TYPE_form_alter().
     *
     * The FAPI element callbacks (such as #process, #element_validate,
     * #value_callback, etc.) used by the widget do not have access to the
     * original $field_definition passed to the widget's constructor. Therefore,
     * if any information is needed from that definition by those callbacks, the
     * widget implementing this method, or a
     * hook_field_widget[_WIDGET_TYPE]_form_alter() implementation, must extract
     * the needed properties from the field definition and set them as ad-hoc
     * $element['#custom'] properties, for later use by its element callbacks.
     *
     * @param \Drupal\Core\Field\FieldItemListInterface $items
     *   Array of default values for this field.
     * @param int $delta
     *   The order of this item in the array of sub-elements (0, 1, 2, etc.).
     * @param array $element
     *   A form element array containing basic properties for the widget:
     *   - #field_parents: The 'parents' space for the field in the form. Most
     *       widgets can simply overlook this property. This identifies the
     *       location where the field values are placed within
     *       $form_state->getValues(), and is used to access processing
     *       information for the field through the getWidgetState() and
     *       setWidgetState() methods.
     *   - #title: The sanitized element label for the field, ready for output.
     *   - #description: The sanitized element description for the field, ready
     *     for output.
     *   - #required: A Boolean indicating whether the element value is required;
     *     for required multiple value fields, only the first widget's values are
     *     required.
     *   - #delta: The order of this item in the array of sub-elements; see $delta
     *     above.
     * @param array $form
     *   The form structure where widgets are being attached to. This might be a
     *   full form structure, or a sub-element of a larger form.
     * @param \Drupal\Core\Form\FormStateInterface $form_state
     *   The current state of the form.
     *
     * @return array
     *   The form elements for a single widget for this field.
     *
     * @see hook_field_widget_form_alter()
     * @see hook_field_widget_WIDGET_TYPE_form_alter()
     */
    public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state)
    {

        $field_name = $items->getFieldDefinition()->getName();
        $display_field_suffix = $this->getSetting('display_field_suffix');


        foreach ($element as $key => $value) {
            $element['physical_quantity_mass'][$key] = $value;
        }

        $element['physical_quantity_mass']['#type'] = 'textfield';
        $element['physical_quantity_mass']['#default_value'] = (isset($items[$delta]->physical_quantity_mass))?$items[$delta]->physical_quantity_mass: NULL;
        $element['physical_quantity_mass']['#attributes']['class'][] = $field_name.'-physical-field';
        $element['physical_quantity_mass']['#element_validate'] = [
            [$this, 'validate'],
        ];

        if($display_field_suffix) {
            $element['physical_quantity_mass']['#field_suffix'] = $this->t('Day(s)');
        }

        return $element;
    }

       /**
        * Validate field
        */
       public function validate($element, FormStateInterface $form_state) {
        $field_data = $element['#value'];
        $field_title = $element['#title'];
        $valid = true;

        if(!is_numeric($field_data)) {
            $valid = false;
        } else {
            if(preg_match("/[a-z]/i", $field_data)){
                $valid = false;
            }
            if($valid) {
                $string_length = strlen($field_data);
                if($string_length > 12) {
                    $valid = false;
                }
            }
        }
        if(!$valid) {
            $error_message = 'Please enter a valid number for '.$field_title;
            $form_state->setError($element, t($error_message));
        }
    }

       /**
        * {@inheritdoc}
        */
       public static function defaultSettings() {

          return [
            'display_field_suffix' => false,
        ] + parent::defaultSettings();
    }

      /**
       * {@inheritdoc}
       */
      public function settingsForm(array $form, FormStateInterface $form_state) {
        $elements = [];

        $elements['display_field_suffix'] = [
            '#type' => 'checkbox',
            '#title' => $this->t('Display field suffix'),
            '#description' => $this->t('Display field suffix on the form element'),
            '#default_value' => $this->getSetting('display_field_suffix')??'false',
        ];

        return $elements;
    }

       /**
        * {@inheritdoc}
        */
       public function settingsSummary() {
        $summary = array();

        $display_field_suffix = ($this->getSetting('display_field_suffix'))?'true':'false';

        $summary_content = 'Display field suffix: '.$display_field_suffix.'<br>';

        // Implement settings summary.
        $summary[] = $this->t($summary_content);

        return $summary;
    }
}